@@ -130,6 +130,11 @@ pub struct SimpleObjectField {
     #[darling(default)]
     pub skip: bool,
     #[darling(default)]
+    pub skip_output: bool,
+    // for InputObject
+    #[darling(default)]
+    pub skip_input: bool,
+    #[darling(default)]
     pub name: Option<String>,
     #[darling(default)]
     pub deprecation: Deprecation,
@@ -361,6 +366,11 @@ pub struct InputObjectField {
     #[darling(default)]
     pub skip: bool,
     #[darling(default)]
+    pub skip_input: bool,
+    // for SimpleObject
+    #[darling(default)]
+    pub skip_output: bool,
+    #[darling(default)]
     pub visible: Option<Visible>,
     #[darling(default)]
     pub secret: bool,
